img {
    max-width: 100%;
}

.titlebox {
    text-align: right;
}


td.label {
    /* Issue #290 */
    background-color: inherit;
}

.footnote-reference {
    /* Issue 290 */
    vertical-align: super;
    font-size: xx-small;
}

.caption {
    /* Issue 292 */
    text-align: center;
    padding-top: 1em;
}

div.figure > img,
div.figure > a > img {
    /* Issue 292 */
    display: block;
    margin-left: auto;
    margin-right: auto;
}

blockquote p, blockquote {
  font-size: 1.25rem;
  font-weight: 300;
  line-height: 1.25;
}

ul.bricks > li {
    display: inline;
    background-color: lightblue;
    padding: 8px;
    border-radius: 5px;
    line-height: 3;
    white-space:nowrap;
    margin: 3px;
}

pre, pre code {
    white-space: pre;
    word-wrap: normal;
    overflow: auto;
}

article.post-micro {
    font-family: Georgia, 'Times New Roman', Times, serif;
    font-size: 1.5em;
}

.image-block {
    display: inline-block;
}

.tags {
    padding-left: 0;
    margin-left: -5px;
    list-style: none;
    text-align: center;

}

.tags > li {
    display: inline-block;
}
.tags > li a {
    display: inline-block;
    padding: .25em .4em;
    font-size: 75%;
    font-weight: 700;
    line-height: 1;
    color: #fff;
    text-align: center;
    white-space: nowrap;
    vertical-align: baseline;
    border-radius: .25rem;
    background-color: #868e96;
}

.tags > li a:hover {
    color: #fff;
    text-decoration: none;
    background-color: #6c757d;
}

.metadata p:before,
.postlist .listdate:after {
    content: " — ";
}

.metadata p:first-of-type:before {
    content: "";
}

.metadata p {
    display: inline;
}

.posttranslations h3 {
    display: inline;
    font-size: 1em;
    font-weight: bold;
}

.posttranslations h3:last-child {
    display: none;
}

.entry-content {
    margin-top: 1em;
}

/* for alignment with Bootstrap's .entry-content styling */
.entry-summary {
    margin-top: 1em;
}

/* Custom page footer */
#footer {
    padding-top: 19px;
    color: #777;
    border-top: 1px solid #e5e5e5;
}

/* hat tip bootstrap/html5 boilerplate */
@media print {
    *, *:before, *:after {
        font-family: Garamond, Junicode, serif;
    }

    body {
        font-size: 12pt;
    }

    article .entry-title a[href]:after,
    article .metadata a[href]:after,
    article .tags a[href]:after {
        content: "";
    }

    article .metadata .sourceline {
        display: none;
    }

    article .metadata .linkline a[href]:after {
        content: " (" attr(href) ")";
    }

    .navbar {
        display: none;
    }
}

pre, .codetable {
    border: 1px solid #ccc;
    border-radius: 0.25rem;
    margin-bottom: 1rem;
}

pre {
    padding: 0.75rem;
}

.codetable tr:first-child td.linenos {
    border-top-left-radius: 0.25rem;
}

.codetable tr:last-child td.linenos {
    border-bottom-left-radius: 0.25rem;
}

.postindexpager {
    padding-bottom: 1rem;
}

ul.navbar-nav {
    margin-top: 0;
}

ul.pager {
    display: flex;
    padding-left: 0;
    list-style: none;
    border-radius: .25rem;
    padding-left: 0;
    margin: 0.5rem 0;
}

ul.pager li.previous {
    margin-right: auto;
    display: inline;
}

ul.pager li.next {
    margin-left: auto;
    display: inline;
}


ul.pager li a {
    display: inline;
    position: relative;
    padding: .5rem .75rem;
    margin-left: -1px;
    line-height: 1.25;
    border: 1px solid #ddd;
    border-radius: .25rem;
}

pre.code {
    white-space: pre-wrap;
}

.byline a:not(:last-child):after {
    content: ",";
}

/* Override incorrect Bootstrap 4 default */
html[dir="rtl"] body {
    text-align: right;
}

/* Set a minimum logo height to ensure .svg are displayed if there is no brand text (#3493) */
#logo {
    min-height: 25px;
}
