# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d min restants per a llegir",
    "(active)": "(actiu)",
    "Also available in:": "També disponible en:",
    "Archive": "Arxiu",
    "Atom feed": "Canal Atom",
    "Authors": "Autors",
    "Categories": "Categories",
    "Comments": "Comentaris",
    "LANGUAGE": "Català",
    "Languages:": "Idiomes:",
    "More posts about %s": "Més entrades sobre %s",
    "Newer posts": "Entrades més recents",
    "Next post": "Entrada següent",
    "Next": "Següent",
    "No posts found.": "Publicació no trobada",
    "Nothing found.": "No trobat",
    "Older posts": "Entrades més antigues",
    "Original site": "Lloc original",
    "Posted:": "Publicat:",
    "Posts about %s": "Entrades sobre %s",
    "Posts by %s": "Entrades per %s",
    "Posts for year %s": "Entrades de l'any %s",
    "Posts for {month_day_year}": "Entrades per {month_day_year}",
    "Posts for {month_year}": "Publicat en {month_year}",
    "Previous post": "Entrada anterior",
    "Previous": "Anterior",
    "Publication date": "Data de publicació",
    "RSS feed": "Canal RSS",
    "Read in English": "Llegeix-ho en català",
    "Read more": "Llegeix-ne més",
    "Skip to main content": "Vés al comentari principal",
    "Source": "Codi",
    "Subcategories:": "Subcategories:",
    "Tags and Categories": "Etiquetes i categories",
    "Tags": "Etiquetes",
    "Toggle navigation": "Commuta la navegació",
    "Uncategorized": "Sense categoria",
    "Up": "Amunt",
    "Updates": "Actualitzacions",
    "Write your page here.": "Escriviu la vostra pàgina aquí.",
    "Write your post here.": "Escriviu la vostra entrada aquí.",
    "old posts, page %d": "entrades antigues, pàgina %d",
    "page %d": "pàgina %d",
    "updated": "actualitzat",
}
