# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "Cần %d phút để đọc",
    "(active)": "(active)",
    "Also available in:": "Cũng có sẵn trong:",
    "Archive": "Kho",
    "Atom feed": "Nguồn cung cấp dữ liệu Atom",
    "Authors": "Tác giả",
    "Categories": "Thể loại",
    "Comments": "Bình luận",
    "LANGUAGE": "Tiếng Việt",
    "Languages:": "Ngôn ngữ:",
    "More posts about %s": "Các bài đăng khác về %s",
    "Newer posts": "Bài viết gần đây",
    "Next post": "Bài viết tiếp theo",
    "Next": "Kế tiếp",
    "No posts found.": "Không tìm thấy bài viết.",
    "Nothing found.": "Không có kết quả.",
    "Older posts": "Bài viết trước đây",
    "Original site": "Trang gốc",
    "Posted:": "Đã đăng：",
    "Posts about %s": "Bài viết về %s",
    "Posts by %s": "Bài đăng bởi %s",
    "Posts for year %s": "Các bài viết trong năm %s",
    "Posts for {month_day_year}": "Các bài đã đăng {month_day_year}",
    "Posts for {month_year}": "Các bài đã đăng {month_year}",
    "Previous post": "Bài viết trước",
    "Previous": "Trước",
    "Publication date": "Ngày phát hành",
    "RSS feed": "Nguồn cung cấp dữ liệu RSS",
    "Read in English": "Phiên bản Tiếng Việt",
    "Read more": "Đọc thêm",
    "Skip to main content": "Chuyển đến nội dung chính",
    "Source": "Nguồn",
    "Subcategories:": "Thể loại con:",
    "Tags and Categories": "Thẻ và Thể loại",
    "Tags": "Thẻ",
    "Toggle navigation": "Chuyển điều hướng",
    "Uncategorized": "Chưa được phân loại",
    "Up": "Trở lên",
    "Updates": "Cập nhật",
    "Write your page here.": "Bắt đầu viết nội dung của trang ở đây.",
    "Write your post here.": "Bắt đầu viết bài ở đây.",
    "old posts, page %d": "các bài viết trước đây, trang %d",
    "page %d": "trang %d",
    "updated": "Đã được cập nhật",
}
