# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d meneti ka toe ki te korero",
    "(active)": "(mātātoa)",
    "Also available in:": "Kei te wātea ano hoki",
    "Archive": "Pūranga",
    "Atom feed": "Atom whangai",
    "Authors": "Kaituhi",
    "Categories": "Kāwai",
    "Comments": "Nga korero",
    "LANGUAGE": "Māori",
    "Languages:": "Reo",
    "More posts about %s": "Ētahi atu pou e pā ana ki %s",
    "Newer posts": "Nga pou hou",
    "Next post": "Pou muri",
    "Next": "Panuku",
    "No posts found.": "Kāore i kitea he korero.",
    "Nothing found.": "Kīhai i kitea",
    "Older posts": "Nga pou tawhito",
    "Original site": "Pae tukutuku",
    "Posted:": "Tukua",
    "Posts about %s": "Tuhinga e pā ana ki %s",
    "Posts by %s": "Tuhinga na %s",
    "Posts for year %s": "Tuhinga mo te tau %s",
    "Posts for {month_day_year}": "Tuhinga mo {month_day_year}",
    "Posts for {month_year}": "Tuhinga mo {month_year}",
    "Previous post": "Tuhinga o mua",
    "Previous": "Whakamuri",
    "Publication date": "Pukapuka tuhi",
    "RSS feed": "Whāngai RSS",
    "Read in English": "Pānuihia i te Maori",
    "Read more": "Pānuihia atu",
    "Skip to main content": "Haere ki te ihirangi matua",
    "Source": "Matapuna",
    "Subcategories:": "Ngā kōpaki",
    "Tags and Categories": "Tūtohu me nga kāwai",
    "Tags": "Tūtohu",
    "Toggle navigation": "Takahuri whakatere",
    "Uncategorized": "Korekorekore",
    "Up": "Ki runga",
    "Updates": "Whakahou",
    "Write your page here.": "Tuhia taau whārangi ki konei.",
    "Write your post here.": "Tuhia to pou i konei.",
    "old posts, page %d": "nga pou tawhito, whārangi %d",
    "page %d": "whārangi %d",
    "updated": "whakahoutia",
}
