# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d mín restante para leitura",
    "(active)": "(ativo)",
    "Also available in:": "Também disponível em:",
    "Archive": "Arquivo",
    "Atom feed": "Feed Atom",
    "Authors": "Autores",
    "Categories": "Categorias",
    "Comments": "Comentários",
    "LANGUAGE": "Português",
    "Languages:": "Idiomas:",
    "More posts about %s": "Mais posts sobre %s",
    "Newer posts": "Posts mais recentes",
    "Next post": "Próximo post",
    "Next": "Próximo",
    "No posts found.": "Nenhum tópico encontrado.",
    "Nothing found.": "Nada encontrado.",
    "Older posts": "Posts mais antigos",
    "Original site": "Site original",
    "Posted:": "Publicado:",
    "Posts about %s": "Posts sobre %s",
    "Posts by %s": "Postado por %s",
    "Posts for year %s": "Posts do ano %s",
    "Posts for {month_day_year}": "Posts do {month_day_year}",
    "Posts for {month_year}": "Posts de {month_year}",
    "Previous post": "Post anterior",
    "Previous": "Anterior",
    "Publication date": "Data de publicação",
    "RSS feed": "Feed RSS",
    "Read in English": "Ler em português",
    "Read more": "Leia mais",
    "Skip to main content": "Pular para o conteúdo principal",
    "Source": "Código",
    "Subcategories:": "Subcategorias:",
    "Tags and Categories": "Tags e Categorias",
    "Tags": "Tags",
    "Toggle navigation": "Alternar navegação",
    "Uncategorized": "Sem categoria",
    "Up": "acima",
    "Updates": "Atualizações",
    "Write your page here.": "Insira a sua página aqui",
    "Write your post here.": "Escreva o seu comentário aqui.",
    "old posts, page %d": "Posts antigos, página %d",
    "page %d": "página %d",
    "updated": "Atualizado",
}
