# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "尚餘 %d 分鐘",
    "(active)": "(啟用)",
    "Also available in:": "其他語言版本：",
    "Archive": "彙整",
    "Atom feed": "",
    "Authors": "作者",
    "Categories": "分類",
    "Comments": "迴響",
    "LANGUAGE": "繁體中文",
    "Languages:": "語言：",
    "More posts about %s": "更多 %s 的文章",
    "Newer posts": "較新的文章",
    "Next post": "下一篇",
    "Next": "下一篇",
    "No posts found.": "沒有找到文章。",
    "Nothing found.": "沒有找到。",
    "Older posts": "較舊的文章",
    "Original site": "原始網站",
    "Posted:": "發佈於：",
    "Posts about %s": "文章分類：%s",
    "Posts by %s": "%s 發佈",
    "Posts for year %s": "%s 年的文章",
    "Posts for {month_day_year}": "{month_day_year}的文章",
    "Posts for {month_year}": "{month_year}的文章",
    "Previous post": "上一篇",
    "Previous": "上一篇",
    "Publication date": "發佈日期",
    "RSS feed": "RSS 訂閱",
    "Read in English": "繁體中文版",
    "Read more": "閱讀更多",
    "Skip to main content": "略過主內容",
    "Source": "原始碼",
    "Subcategories:": "子分類",
    "Tags and Categories": "標籤和分類",
    "Tags": "標籤",
    "Toggle navigation": "切換導航",
    "Uncategorized": "未分類",
    "Up": "向上",
    "Updates": "更新",
    "Write your page here.": "從這裡開始編輯頁面",
    "Write your post here.": "從這裡開始編輯文章",
    "old posts, page %d": "舊文章，第 %d 頁",
    "page %d": "第 %d 頁",
    "updated": "",
}
