# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d perc van hátra olvasni",
    "(active)": "(aktív)",
    "Also available in:": "Olvasható még:",
    "Archive": "Archív",
    "Atom feed": "Atom",
    "Authors": "Szerzők",
    "Categories": "Kategóriák",
    "Comments": "Hozzászólások",
    "LANGUAGE": "Magyarul",
    "Languages:": "Nyelvek:",
    "More posts about %s": "Több bejegyzés erről: %s",
    "Newer posts": "Újabb bejegyzések",
    "Next post": "A következő bejegyzés",
    "Next": "Következő",
    "No posts found.": "Nincs ilyen bejegyzés.",
    "Nothing found.": "Nincs találat.",
    "Older posts": "Régebbi bejegyzések",
    "Original site": "Eredeti honlap",
    "Posted:": "Megjelent:",
    "Posts about %s": "Bejegyzések erről: %s",
    "Posts by %s": "Bejegyzések %s által",
    "Posts for year %s": "%s. bejegyzések",
    "Posts for {month_day_year}": "{month_day_year} bejegyzések",
    "Posts for {month_year}": "{month_year} bejegyzések",
    "Previous post": "Az előző bejegyzés ",
    "Previous": "Előző",
    "Publication date": "A megjelenés dátuma",
    "RSS feed": "RSS",
    "Read in English": "Olvass magyarul",
    "Read more": "Olvass tovább",
    "Skip to main content": "Ugrás a főtartalomra",
    "Source": "Forrás",
    "Subcategories:": "Alkategóriák:",
    "Tags and Categories": "Címkék és kategóriák",
    "Tags": "Címkék",
    "Toggle navigation": "",
    "Uncategorized": "Nincs kategorizálva",
    "Up": "Fel",
    "Updates": "Frissítések",
    "Write your page here.": "Ide írd az oldalad.",
    "Write your post here.": "Ide írd a bejegyzésed.",
    "old posts, page %d": "régi bejegyzések, %d. oldal",
    "page %d": "%d. oldal",
    "updated": "",
}
