# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d నిమిషాలు చదవడానికి కావలెను ",
    "(active)": "(క్రియాశీల)",
    "Also available in:": "ఇందులో కూడా లభించును:",
    "Archive": "అభిలేఖలు ",
    "Atom feed": "అటామ్ ఫీడ్",
    "Authors": "రచయితలు",
    "Categories": "వర్గాలు",
    "Comments": "వ్యాఖ్యలు",
    "LANGUAGE": "తెలుగు",
    "Languages:": "భాషలు:",
    "More posts about %s": "%s గూర్చి మరిన్ని టపాలు",
    "Newer posts": "కొత్త టపాలు",
    "Next post": "తరువాతి టపా",
    "Next": "తరువాతి",
    "No posts found.": "టపాలు ఏవీ కనుగొనబడలేదు.",
    "Nothing found.": "ఏదీ కనుగొనబడలేదు.",
    "Older posts": "పాత టపాలు",
    "Original site": "వాస్తవ సైట్",
    "Posted:": "ప్రచురుంచిన తేదీ:",
    "Posts about %s": "%s గూర్చి టపాలు",
    "Posts by %s": "%s యొక్క టపాలు",
    "Posts for year %s": "%s సంవత్సర టపాలు",
    "Posts for {month_day_year}": "{month_day_year} యొక్క టపాలు",
    "Posts for {month_year}": "{month_year} యొక్క టపాలు",
    "Previous post": "మునుపటి టపా",
    "Previous": "మునుపటి",
    "Publication date": "ప్రచురణ తేదీ",
    "RSS feed": "RSS ఫీడ్",
    "Read in English": "తెలుగులో చదవండి",
    "Read more": "ఇంకా చదవండి",
    "Skip to main content": "ప్రధాన విషయానికి వెళ్ళు",
    "Source": "మూలం",
    "Subcategories:": "ఉపవర్గాలు:",
    "Tags and Categories": "ట్యాగ్లు మరియు వర్గాలు",
    "Tags": "ట్యాగ్లు",
    "Toggle navigation": "నావిగేషన్‌ను టోగుల్ చేయండి",
    "Uncategorized": "వర్గీకరించని",
    "Up": "పైకి",
    "Updates": "నవీకరణలు",
    "Write your page here.": "మీ పేజీ ఇక్కడ రాయండి.",
    "Write your post here.": "ఇక్కడ మీ టపా ను వ్రాయండి.",
    "old posts, page %d": "పాత టపాలు, పేజీ %d",
    "page %d": "పేజీ %d",
    "updated": "నవీకరించబడింది",
}
