# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d минути до прочитане",
    "(active)": "(активно)",
    "Also available in:": "Достъпно също на:",
    "Archive": "Архив",
    "Atom feed": "",
    "Authors": "Автори",
    "Categories": "Категории",
    "Comments": "Коментари",
    "LANGUAGE": "Български",
    "Languages:": "Езици:",
    "More posts about %s": "Още публикации относно %s",
    "Newer posts": "Нови публикации",
    "Next post": "Следваща публикация",
    "Next": "",
    "No posts found.": "Не са намерени публикации.",
    "Nothing found.": "Нищо не е намерено.",
    "Older posts": "Стари публикации",
    "Original site": "Оригиналния сайт",
    "Posted:": "Публикyвано:",
    "Posts about %s": "Публикации относно %s",
    "Posts by %s": "Публикации от %s",
    "Posts for year %s": "Публикации за %s година",
    "Posts for {month_day_year}": "Публикации от {month_day_year}",
    "Posts for {month_year}": "Публикации за {month_year}",
    "Previous post": "Предишна публикация",
    "Previous": "",
    "Publication date": "Дата на публикуване",
    "RSS feed": "RSS поток",
    "Read in English": "Прочетете на български",
    "Read more": "Чети нататък",
    "Skip to main content": "Прескочи до основното съдържание",
    "Source": "Изходен код",
    "Subcategories:": "Подкатегории:",
    "Tags and Categories": "Тагове и Категории",
    "Tags": "Тагове",
    "Toggle navigation": "",
    "Uncategorized": "Без категория",
    "Up": "",
    "Updates": "Обновления",
    "Write your page here.": "Напиши тук текста на твоята страница.",
    "Write your post here.": "Напиши тук текста на твоята публикация.",
    "old posts, page %d": "стари публикации, страница %d",
    "page %d": "страница %d",
    "updated": "",
}
