# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d min demorantas de lectura",
    "(active)": "(actiu)",
    "Also available in:": "Tanben disponibla en :",
    "Archive": "Archius",
    "Atom feed": "Flux Atom",
    "Authors": "Autors",
    "Categories": "Categorias",
    "Comments": "Comentaris",
    "LANGUAGE": "Occitan",
    "Languages:": "Lengas :",
    "More posts about %s": "Mai de publicacion sus %s",
    "Newer posts": "Publicacions mai recentas",
    "Next post": "Publicacion seguenta",
    "Next": "Seguent",
    "No posts found.": "Cap de publicacion pas trobada.",
    "Nothing found.": "Res pas trobat.",
    "Older posts": "Publicacions mai ancianas",
    "Original site": "Site original",
    "Posted:": "Publicada lo :",
    "Posts about %s": "Publicacions tocant %s",
    "Posts by %s": "Publicat per %s",
    "Posts for year %s": "Publicacions per annada %s",
    "Posts for {month_day_year}": "Publicacions per {month_day_year}",
    "Posts for {month_year}": "Publicacions per {month_year} ",
    "Previous post": "Publicacion precedenta",
    "Previous": "Precedent",
    "Publication date": "Data de publicacion",
    "RSS feed": "Flux RSS",
    "Read in English": "Legir en occitan",
    "Read more": "Ne legir mai",
    "Skip to main content": "Passar al contengut màger",
    "Source": "Font",
    "Subcategories:": "Jos-categorias :",
    "Tags and Categories": "Etiquetas e categorias",
    "Tags": "Etiquetas",
    "Toggle navigation": "Alternar la navigacion",
    "Uncategorized": "Sens categoria",
    "Up": "Amont",
    "Updates": "Actualizacions",
    "Write your page here.": "Escrivètz vòstra pagina aquí.",
    "Write your post here.": "Escrivètz vòstra publicacion aquí.",
    "old posts, page %d": "publicacions ancianas, pagina %d",
    "page %d": "pagina %d",
    "updated": "actualizada",
}
