# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d minuuttia lukuaikaa",
    "(active)": "(aktiivinen)",
    "Also available in:": "Saatavilla myös:",
    "Archive": "Arkisto",
    "Atom feed": "Atom-syöte",
    "Authors": "Kirjoittajat",
    "Categories": "Kategoriat",
    "Comments": "Kommentit",
    "LANGUAGE": "Suomi",
    "Languages:": "Kielet:",
    "More posts about %s": "Lisää kirjoituksia aiheesta %s",
    "Newer posts": "Uudempia kirjoituksia",
    "Next post": "Seuraava kirjoitus",
    "Next": "Seuraava",
    "No posts found.": "Kirjoituksia ei löytynyt.",
    "Nothing found.": "Ei hakutuloksia.",
    "Older posts": "Vanhempia kirjoituksia",
    "Original site": "Alkuperäinen sivusto",
    "Posted:": "Kirjoitettu:",
    "Posts about %s": "Kirjoituksia aiheesta %s",
    "Posts by %s": "Artikkelit kirjoittajalta %s",
    "Posts for year %s": "Kirjoituksia vuodelta %s",
    "Posts for {month_day_year}": "Kirjoituksia ajalta {month_day_year}",
    "Posts for {month_year}": "Kirjoituksia ajalta {month_year}",
    "Previous post": "Edellinen kirjoitus",
    "Previous": "Edellinen",
    "Publication date": "Julkaisupäivämäärä",
    "RSS feed": "RSS-syöte",
    "Read in English": "Lue suomeksi",
    "Read more": "Lue lisää",
    "Skip to main content": "Hyppää sisältöön",
    "Source": "Lähde",
    "Subcategories:": "Alakategoriat:",
    "Tags and Categories": "Avainsanat ja kategoriat",
    "Tags": "Avainsanat",
    "Toggle navigation": "Vaihda navigointia",
    "Uncategorized": "Luokittelematon",
    "Up": "Ylös",
    "Updates": "Päivitykset",
    "Write your page here.": "Kirjoita sisältö tähän.",
    "Write your post here.": "Kirjoita sisältö tähän.",
    "old posts, page %d": "vanhoja kirjoituksia, sivu %d",
    "page %d": "sivu %d",
    "updated": "päivitetty",
}
