# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d دقیقه برای خواندن باقی مانده",
    "(active)": "(فعال)",
    "Also available in:": "همچنین قابل دسترس از:",
    "Archive": "آرشیو",
    "Atom feed": "",
    "Authors": "نویسنده‌ها",
    "Categories": "دسته‌ها",
    "Comments": "دیدگاه‌‌‌ها",
    "LANGUAGE": "فارسی",
    "Languages:": "زبان‌‌ها:",
    "More posts about %s": "ارسال‌های بیشتر دربارهٔ%s",
    "Newer posts": "ارسال‌های جدید‌تر",
    "Next post": "ارسال بعدی",
    "Next": "بعدی",
    "No posts found.": "هیچ پستی پیدا نشد.",
    "Nothing found.": "هیچ‌چیزی پیدا نشد.",
    "Older posts": "پست‌های قدیمی‌تر",
    "Original site": "سایت اصلی",
    "Posted:": "ارسال شده:",
    "Posts about %s": "ارسال‌ها دربارهٔ %s",
    "Posts by %s": "ارسال‌های %s",
    "Posts for year %s": "ارسال‌ها برای سال %s",
    "Posts for {month_day_year}": "ارسال برای {month_day_year}",
    "Posts for {month_year}": "ارسال برای {month_year}",
    "Previous post": "ارسال پیشین",
    "Previous": "قبلی",
    "Publication date": "تاریخ انتشار",
    "RSS feed": "خوراک",
    "Read in English": "به فارسی بخوانید",
    "Read more": "بیشتر بخوانید",
    "Skip to main content": "متن اصلی را نادیده بگیر",
    "Source": "منبع",
    "Subcategories:": "زیر بخش‌ها:",
    "Tags and Categories": "برچسب‌ها و دسته‌ها",
    "Tags": "برچسب‌ها",
    "Toggle navigation": "",
    "Uncategorized": "دسته‌بندی نشده",
    "Up": "",
    "Updates": "بروزرسانی‌ها",
    "Write your page here.": "من صفحه را این‌جا بنویسید. ",
    "Write your post here.": "متن پست‌تان را این‌جا بنویسید.",
    "old posts, page %d": "صفحهٔ ارسال‌های قدیمی %d",
    "page %d": "برگه %d",
    "updated": "",
}
