# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "पढ़ने के लिए %d मिनट शेष",
    "(active)": "(सक्रिय)",
    "Also available in:": "उपलब्ध भाषाएँ:",
    "Archive": "संग्रह",
    "Atom feed": "एटम फीड",
    "Authors": "लेखक",
    "Categories": "श्रेणियाँ",
    "Comments": "टिप्पणियाँ",
    "LANGUAGE": "हिन्दी",
    "Languages:": "भाषाएँ:",
    "More posts about %s": "%s के बारे में और पोस्ट",
    "Newer posts": "नई पोस्ट",
    "Next post": "अगला पोस्ट",
    "Next": "अगला",
    "No posts found.": "कोई पोस्ट नहीं मिला",
    "Nothing found.": "कुछ नहीं मिला",
    "Older posts": "पुरानी पोस्ट",
    "Original site": "मूल साइट",
    "Posted:": "पोस्टेड:",
    "Posts about %s": "%s के बारे में पोस्ट",
    "Posts by %s": "%s द्वारा पोस्ट",
    "Posts for year %s": "वर्ष  %s के पोस्ट",
    "Posts for {month_day_year}": "{month_day_year}  के पोस्ट",
    "Posts for {month_year}": "{month_year}  के पोस्ट",
    "Previous post": "पिछली पोस्ट",
    "Previous": "पिछला",
    "Publication date": "प्रकाशन तिथि",
    "RSS feed": "आर एस एस फ़ीड",
    "Read in English": "हिन्दी में पढ़िए",
    "Read more": "और पढ़िए",
    "Skip to main content": "मुख्य विषयवस्तु में जाएं",
    "Source": "स्रोत",
    "Subcategories:": "उपश्रेणी",
    "Tags and Categories": "टैग्स और श्रेणियाँ",
    "Tags": "टैग्स",
    "Toggle navigation": "टॉगल नेविगेशन",
    "Uncategorized": "अवर्गीकृत",
    "Up": "ऊपर",
    "Updates": "अपडेट्स",
    "Write your page here.": "अपना पेज यहाँ लिखें",
    "Write your post here.": "अपनी पोस्ट यहाँ लिखें",
    "old posts, page %d": "पुराने पोस्ट, पृष्‍ठ %d",
    "page %d": "पृष्‍ठ %d",
    "updated": "संशोधित",
}
