# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d min oor om te lees",
    "(active)": "(aktief)",
    "Also available in:": "Ook beskikbaar in:",
    "Archive": "Argief",
    "Atom feed": "Atom-voer",
    "Authors": "Outeurs",
    "Categories": "Kategorieë",
    "Comments": "Opmerkings",
    "LANGUAGE": "Afrikaans",
    "Languages:": "Tale:",
    "More posts about %s": "Meer plasings oor %s",
    "Newer posts": "Jonger plasings",
    "Next post": "Volgende plasing",
    "Next": "Volgende",
    "No posts found.": "Geen plasings gevind nie.",
    "Nothing found.": "Niks gevind nie.",
    "Older posts": "Ouer plasings",
    "Original site": "Oorspronklike werf",
    "Posted:": "Geplaas:",
    "Posts about %s": "Plasings oor %s",
    "Posts by %s": "Plasings deur %s",
    "Posts for year %s": "Plasings vir %s",
    "Posts for {month_day_year}": "Plasings vir {month_day_year}",
    "Posts for {month_year}": "Plasings vir {month_year}",
    "Previous post": "Vorige plasing",
    "Previous": "Vorige",
    "Publication date": "Publikasiedatum",
    "RSS feed": "RSS-voer",
    "Read in English": "Lees in Afrikaans",
    "Read more": "Lees meer",
    "Skip to main content": "Spring na die hoofinhoud",
    "Source": "Bron",
    "Subcategories:": "Subkategorieë:",
    "Tags and Categories": "Etikette en kategorieë",
    "Tags": "Etikette",
    "Toggle navigation": "Wissel navigasie",
    "Uncategorized": "Ongekategoriseerd",
    "Up": "Op",
    "Updates": "Bywerkings",
    "Write your page here.": "Skryf die bladsy hier.",
    "Write your post here.": "Skryf die plasing hier.",
    "old posts, page %d": "ou plasings, bladsy %d",
    "page %d": "bladsy %d",
    "updated": "bygewerk",
}
