div.prompt {
  padding: 0.6em;
  font-size: 13px;
  background-color: #E9E9E9;
  margin-right: 1em;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}

div.output_prompt {
  /* 5px right shift to account for margin in parent container */
  margin: 0 5px 0 0px;
}

div.output_area pre {
  font-size: 13px;
}

div.text_cell_render {
  padding: 0px;
  color: #333333;
}

.rendered_html p {
  text-align: left;
}

.rendered_html ul {
  margin: 0 0 12px 25px;
}

.rendered_html :visited {
  text-decoration: none;
}

.rendered_html :link {
  text-decoration: none;
}

.rendered_html pre, .rendered_html code {
  background-color: #DDDDDD;
  margin: 1em 0em;
  font-size: 14px;
}

.rendered_html pre {
  padding-left: 0.5em;
  padding-right: 0.5em;
  padding-top: 0.05em;
  padding-bottom: 0.05em;
}

.page-content > .content p {
  margin: 0 0 0px;
}
