# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "a restin altris %d minûts di leture",
    "(active)": "(atîf)",
    "Also available in:": "Disponibil ancje in:",
    "Archive": "Archivi",
    "Atom feed": "Feed Atom",
    "Authors": "Autôrs",
    "Categories": "Categoriis",
    "Comments": "Coments",
    "LANGUAGE": "Furlan",
    "Languages:": "Lenghis:",
    "More posts about %s": "Altris articui su %s",
    "Newer posts": "Articui plui resints",
    "Next post": "Prossim articul",
    "Next": "Prossim",
    "No posts found.": "Nissun articul cjatât.",
    "Nothing found.": "Nol è stât cjatât nuie.",
    "Older posts": "Articui precedents",
    "Original site": "Sît originâl",
    "Posted:": "Publicât:",
    "Posts about %s": "Articui su %s",
    "Posts by %s": "Articui di %s",
    "Posts for year %s": "Articui par an %s",
    "Posts for {month_day_year}": "Articui par {month_day_year}",
    "Posts for {month_year}": "Articui par {month_year}",
    "Previous post": "Articul precedent",
    "Previous": "Precedent",
    "Publication date": "Date di publicazion",
    "RSS feed": "Feed RSS",
    "Read in English": "Lei par furlan",
    "Read more": "Continue la leture",
    "Skip to main content": "Va al test principâl",
    "Source": "Document origjinâl",
    "Subcategories:": "Subcategoriis:",
    "Tags and Categories": "Tags e categoriis",
    "Tags": "Tags",
    "Toggle navigation": "Ativâ la navigazion",
    "Uncategorized": "Cence categorie",
    "Up": "Su",
    "Updates": "Inzornaments",
    "Write your page here.": "Scrîf ca la tô pagjine.",
    "Write your post here.": "Scrîf ca il to articul.",
    "old posts, page %d": "articui vecjos, pagjine %d",
    "page %d": "pagjine %d",
    "updated": "inzornât",
}
