# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "残りを読むのに必要な時間は%d分",
    "(active)": "(有効)",
    "Also available in:": "他の言語で読む：",
    "Archive": "過去記事一覧",
    "Atom feed": "Atomフィード",
    "Authors": "著者一覧",
    "Categories": "カテゴリー",
    "Comments": "コメント",
    "LANGUAGE": "日本語",
    "Languages:": "言語：",
    "More posts about %s": "%sに関する記事一覧",
    "Newer posts": "新しい記事",
    "Next post": "次の記事",
    "Next": "次へ",
    "No posts found.": "記事はありません。",
    "Nothing found.": "なにも見つかりませんでした。",
    "Older posts": "過去の記事",
    "Original site": "元のサイト",
    "Posted:": "公開日時：",
    "Posts about %s": "%sについての記事",
    "Posts by %s": "%sの記事一覧",
    "Posts for year %s": "%s年の記事",
    "Posts for {month_day_year}": "{month_day_year}の記事",
    "Posts for {month_year}": "{month_year}の記事",
    "Previous post": "一つ前の記事",
    "Previous": "前へ",
    "Publication date": "公開日",
    "RSS feed": "RSSフィード",
    "Read in English": "日本語で読む",
    "Read more": "続きを読む",
    "Skip to main content": "本文を読み飛ばす",
    "Source": "ソース",
    "Subcategories:": "サブカテゴリー",
    "Tags and Categories": "カテゴリーおよびタグ一覧",
    "Tags": "タグ",
    "Toggle navigation": "ナビゲーションを隠す",
    "Uncategorized": "カテゴリーなし",
    "Up": "上へ",
    "Updates": "更新",
    "Write your page here.": "ここに記事を記述してください。",
    "Write your post here.": "ここに記事を記述してください。",
    "old posts, page %d": "過去の記事 %dページ目",
    "page %d": "ページ%d",
    "updated": "更新日時",
}
