# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d minuter kvar att läsa",
    "(active)": "(aktiv)",
    "Also available in:": "Även tillgänglig på:",
    "Archive": "Arkiv",
    "Atom feed": "",
    "Authors": "",
    "Categories": "Kategorier",
    "Comments": "Kommentarer",
    "LANGUAGE": "Svenska",
    "Languages:": "Språk:",
    "More posts about %s": "Fler inlägg om %s",
    "Newer posts": "Nya inlägg",
    "Next post": "Nästa inlägg",
    "Next": "",
    "No posts found.": "Inga inlägg hittade.",
    "Nothing found.": "Inget hittat.",
    "Older posts": "Äldre inlägg",
    "Original site": "Originalsida",
    "Posted:": "Publicerad:",
    "Posts about %s": "Inlägg om %s",
    "Posts by %s": "",
    "Posts for year %s": "Inlägg för år %s",
    "Posts for {month_day_year}": "Inlägg för {month_day_year}",
    "Posts for {month_year}": "Inlägg för {month_year}",
    "Previous post": "Föregående inlägg",
    "Previous": "",
    "Publication date": "Publiceringsdatum",
    "RSS feed": "RSS-flöde",
    "Read in English": "Läs på svenska",
    "Read more": "Läs mer",
    "Skip to main content": "Hoppa till huvudinnehåll",
    "Source": "Källa",
    "Subcategories:": "Underkategorier:",
    "Tags and Categories": "Taggar och Kategorier",
    "Tags": "Taggar",
    "Toggle navigation": "",
    "Uncategorized": "",
    "Up": "",
    "Updates": "",
    "Write your page here.": "Skriv din sida här.",
    "Write your post here.": "Skriv ditt inlägg här.",
    "old posts, page %d": "gamla inlägg, sida %d",
    "page %d": "sida %d",
    "updated": "",
}
