# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "Залишилось читати %d хвилин",
    "(active)": "(активне)",
    "Also available in:": "Іншою мовою:",
    "Archive": "Архів",
    "Atom feed": "Стрічка у форматі Atom",
    "Authors": "Автори",
    "Categories": "Категорії",
    "Comments": "Коментарі",
    "LANGUAGE": "Українська",
    "Languages:": "Мови:",
    "More posts about %s": "Більше статей про  %s",
    "Newer posts": "Новіші статті",
    "Next post": "Наступна стаття",
    "Next": "Вперед",
    "No posts found.": "Не знайдено жодної статті",
    "Nothing found.": "Нічого не знайдено",
    "Older posts": "Старіші статті",
    "Original site": "Оригінал сайту",
    "Posted:": "Опублікована:",
    "Posts about %s": "Статті про %s",
    "Posts by %s": "Статті %s",
    "Posts for year %s": "Статті за %s рік",
    "Posts for {month_day_year}": "Статті за {month_day_year}",
    "Posts for {month_year}": "Статті за {month_year}",
    "Previous post": "Попередня стаття",
    "Previous": "Назад",
    "Publication date": "Дата публікації",
    "RSS feed": "Стрічка у форматі RSS",
    "Read in English": "Читати українською",
    "Read more": "Читати далі",
    "Skip to main content": "Перейти до основного матеріалу",
    "Source": "Джерело",
    "Subcategories:": "Підкатегорії:",
    "Tags and Categories": "Теги і категорії",
    "Tags": "Теги",
    "Toggle navigation": "Перемкнути навігацію",
    "Uncategorized": "Без категорії",
    "Up": "Нагору",
    "Updates": "Підписки",
    "Write your page here.": "Напишіть Вашу сторінку тут.",
    "Write your post here.": "Напишить Вашу статтю тут.",
    "old posts, page %d": "старі статті, сторінка %d",
    "page %d": "сторінка %d",
    "updated": "оновлено",
}
