# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "वाचण्यासाठी %d मिनिटे शिल्लक आहेत",
    "(active)": "(सक्रिय)",
    "Also available in:": "मध्ये देखील उपलब्ध:",
    "Archive": "संग्रह",
    "Atom feed": "अ‍ॅटम फीड",
    "Authors": "लेखक",
    "Categories": "श्रेणी",
    "Comments": "टिप्पण्या",
    "LANGUAGE": "मराठी",
    "Languages:": "भाषा:",
    "More posts about %s": "%s बद्दल अधिक पोस्ट",
    "Newer posts": "नवीन पोस्ट्स",
    "Next post": "पुढील पोस्ट",
    "Next": "पुढे",
    "No posts found.": "कोणतीही पोस्ट आढळली नाहीत",
    "Nothing found.": "काहीही सापडले नाही.",
    "Older posts": "जुने पोस्ट",
    "Original site": "मूळ साइट",
    "Posted:": "पोस्ट केले:",
    "Posts about %s": "%s बद्दल पोस्ट",
    "Posts by %s": "%s द्वारा पोस्ट केलेले",
    "Posts for year %s": "%s वर्षासाठी पोस्ट",
    "Posts for {month_day_year}": "{month_day_year} साठी पोस्ट्स",
    "Posts for {month_year}": "{month_year} साठी पोस्ट्स",
    "Previous post": "मागील पोस्ट",
    "Previous": "मागील",
    "Publication date": "प्रकाशन तारीख",
    "RSS feed": "आरएसएस फीड",
    "Read in English": "मराठी मध्ये वाचा",
    "Read more": "अधिक वाचा",
    "Skip to main content": "मुख्य सामग्रीकडे जा",
    "Source": "स्रोत",
    "Subcategories:": "उपश्रेणी:",
    "Tags and Categories": "टॅग्ज आणि श्रेण्या",
    "Tags": "टॅग्ज",
    "Toggle navigation": "नेव्हिगेशन टॉगल करा",
    "Uncategorized": "अवर्गीकृत",
    "Up": "वर",
    "Updates": "अद्यतने",
    "Write your page here.": "आपले पृष्ठ येथे लिहा.",
    "Write your post here.": "आपले पोस्ट येथे लिहा.",
    "old posts, page %d": "जुने पोस्ट, पृष्ठ %d",
    "page %d": "पृष्ठ %d",
    "updated": "अद्यतनित",
}
