# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%dminutas de lectura remanente",
    "(active)": "(active)",
    "Also available in:": "Anque disponibile in:",
    "Archive": "Archivo",
    "Atom feed": "Fluxo Atom",
    "Authors": "Authores",
    "Categories": "Categorias",
    "Comments": "Commentos",
    "LANGUAGE": "Interlingua",
    "Languages:": "Linguas:",
    "More posts about %s": "Plure entratas super %s",
    "Newer posts": "Entratas plus recente",
    "Next post": "Entrata successive",
    "Next": "Successive",
    "No posts found.": "Nulle entrata esseva trovate.",
    "Nothing found.": "Nihil esseva trovate.",
    "Older posts": "Entratas plus vetule",
    "Original site": "Sito original",
    "Posted:": "Publicate:",
    "Posts about %s": "Entratas super %s",
    "Posts by %s": "Entratas per %s",
    "Posts for year %s": "Entratas del anno %s",
    "Posts for {month_day_year}": "Entratas de {month_day_year}",
    "Posts for {month_year}": "Entratas de {month_year}",
    "Previous post": "Entrata precedente",
    "Previous": "Precendente",
    "Publication date": "Data de publication",
    "RSS feed": "Fluxo RSS",
    "Read in English": "Lege in interlingua",
    "Read more": "Lege plus",
    "Skip to main content": "Salta al contento principal",
    "Source": "Sorgente",
    "Subcategories:": "Subcategorias:",
    "Tags and Categories": "Etiquettas e categorias",
    "Tags": "Etiquettas",
    "Toggle navigation": "Commuta navigation",
    "Uncategorized": "Sin categoria",
    "Up": "In alto",
    "Updates": "Actualisationes",
    "Write your page here.": "Scribe tu pagina hic.",
    "Write your post here.": "Scribe tu entrata hic.",
    "old posts, page %d": "Vetule entratas, pagina %d",
    "page %d": "pagina %d",
    "updated": "actualisate",
}
